@@ -1,8 +1,7 @@
-"""Functions for dealing with inputs and outputs from Grasshopper components."""
+"""Functions for managing the setting of Rhino's IronPython path."""
 import os
 import io
 import xml.etree.ElementTree
-import plistlib
 
 try:
     from ladybug.futil import nukedir, copy_file_tree
@@ -15,9 +14,11 @@ PACKAGES = \
     ('ladybug_rhino', 'ladybug_geometry', 'ladybug_geometry_polyskel',
      'ladybug', 'ladybug_comfort', 'honeybee', 'honeybee_standards', 'honeybee_energy',
      'honeybee_radiance', 'honeybee_radiance_folder', 'honeybee_radiance_command',
-     'dragonfly', 'dragonfly_energy', 'dragonfly_radiance')
+     'dragonfly', 'dragonfly_energy', 'dragonfly_radiance', 'dragonfly_uwg')
 # Rhino versions that the plugins are compatible with
 RHINO_VERSIONS = ('6.0', '7.0')
+# UUID that McNeel uses to identify the IronPython plugin
+IRONPYTHON_ID = '814d908a-e25c-493d-97e9-ee3861957f49'
 
 
 def create_python_package_dir():
@@ -58,37 +59,49 @@ def iron_python_search_path(python_package_dir, settings_file=None,
     new_settings = []
     if os.name == 'nt':  # we are on Windows
         all_settings = [settings_file] if settings_file is not None else \
-            find_installed_settings_windows()
+            find_ironpython_settings_windows()
         for sf in all_settings:
             dest_settings = iron_python_search_path_windows(
                 python_package_dir, sf, destination_file)
             new_settings.append(dest_settings)
     else:  # we are on Mac, Linux, or some other unix-based system
-        # TODO: replace this with iron_python_search_path_mac when McNeel makes it work
         copy_packages_to_rhino_scripts(python_package_dir)
     return new_settings
 
 
-def find_installed_settings_windows():
+def find_installed_rhino_versions_windows():
+    """Get a list of the compatible Rhino versions installed on this Windows machine."""
+    program_files = os.getenv('PROGRAMFILES')
+    installed_vers = []
+    for ver in RHINO_VERSIONS:
+        rhino_path = os.path.join(
+            program_files, 'Rhino {}'.format(ver[0]), 'System', 'Rhino.exe')
+        if os.path.isfile(rhino_path):
+            installed_vers.append(ver)
+    return installed_vers
+
+
+def find_ironpython_settings_windows():
     """Get a list of all settings XML files for the supported RHINO_VERSIONS."""
     installed_set_files = []
     appdata_roaming = os.getenv('APPDATA')
-    for ver in RHINO_VERSIONS:
-        plugin_folder = os.path.join(appdata_roaming, 'McNeel',
-                                     'Rhinoceros', ver, 'Plug-ins')
-        if os.path.isdir(plugin_folder):
-            ip_path = None
-            for plugin in os.listdir(plugin_folder):
-                if plugin.startswith('IronPython'):
-                    ip_path = os.path.join(plugin_folder, plugin)
-                    break
-            if ip_path is not None:
-                settings_path = os.path.join(ip_path, 'settings')
-                if os.path.isdir(settings_path):
-                    for set_file in os.listdir(settings_path):
-                        if set_file.startswith('settings-Scheme'):
-                            sf = os.path.join(settings_path, set_file)
-                            installed_set_files.append(sf)
+    for ver in find_installed_rhino_versions_windows():
+        # get the settings folder or create it if it doesn't exist
+        plugin_folder = os.path.join(
+            appdata_roaming, 'McNeel', 'Rhinoceros', ver, 'Plug-ins')
+        settings_path = os.path.join(
+            plugin_folder, 'IronPython ({})'.format(IRONPYTHON_ID), 'settings')
+        if not os.path.isdir(settings_path):
+            os.makedirs(settings_path)
+        # append the default settings to the list of files to edit
+        sf = os.path.join(settings_path, 'settings-Scheme__Default.xml')
+        installed_set_files.append(sf)
+        # get the search paths for any Rhino-inside instances if they exist
+        for set_file in os.listdir(settings_path):
+            if set_file.startswith('settings-Scheme') and \
+                    set_file != 'settings-Scheme__Default.xml':
+                sf = os.path.join(settings_path, set_file)
+                installed_set_files.append(sf)
     return installed_set_files
 
 
@@ -171,60 +184,12 @@ def iron_python_search_path_windows(python_package_dir, settings_file,
     return destination_file
 
 
-def iron_python_search_path_mac(python_package_dir, settings_file=None,
-                                destination_file=None):
-    """Set Rhino to search for libraries in a given directory (on Mac).
-
-    This is used as part of the installation process to ensure that Grasshopper
-    looks for the core Python libraries in the ladybug_tools folder. The file
-    will not be edited if the python_package_dir is already in the settings file.
-
-    Args:
-        python_package_dir: The path to a directory that contains the Ladybug
-            Tools core libraries.
-        settings_file: An optional .plist settings file to which the python_package_dir
-            will be added. If None, this method will search the current user's
-            Library/Preferences folder for the default location of this file.
-        destination_file: Optional destination file to write out the edited settings
-            file. If it is None, the settings_file will be overwritten.
-    """
-    # find the path to the IronPython plugin
-    if settings_file is None:
-        home_folder = os.getenv('HOME') or os.path.expanduser('~')
-        plugin_folder = os.path.join(home_folder, 'Library', 'Preferences')
-        settings_file = os.path.join(plugin_folder, 'com.mcneel.rhinoceros.plist')
-
-    # load the plist file and check the search paths
-    sp_key = 'User.Plug-Ins.814d908a-e25c-493d-97e9-ee3861957f49.Settings.SearchPaths'
-    with open(settings_file, 'rb') as fp:
-        pl = plistlib.load(fp)
-    search_path_needed = False
-    existing_paths = None
-    try:
-        if python_package_dir not in pl[sp_key]:
-            search_path_needed = True  # the key is there but not our package path
-            existing_paths = pl[sp_key]
-    except KeyError:  # the key isn't there and we must add it
-        search_path_needed = True
-
-    # add the search paths if it was not found
-    if destination_file is None:
-        destination_file = settings_file
-    if search_path_needed:
-        new_paths = '{};{}'.format(existing_paths, python_package_dir) \
-            if existing_paths is not None else python_package_dir
-        pl[sp_key] = new_paths
-        with open(destination_file, 'wb') as fp:
-            plistlib.dump(pl, fp, fmt=plistlib.FMT_BINARY)
-    return destination_file
-
-
 def find_installed_rhino_scripts():
     """Get the path to the current user's Rhino scripts folder if it exists."""
     installed_scripts = []
     if os.name == 'nt':  # we are on Windows
         appdata_roaming = os.getenv('APPDATA')
-        for ver in RHINO_VERSIONS:
+        for ver in find_installed_rhino_versions_windows():
             scripts_folder = os.path.join(appdata_roaming, 'McNeel',
                                           'Rhinoceros', ver, 'scripts')
             if os.path.isdir(scripts_folder):
@@ -234,8 +199,9 @@ def find_installed_rhino_scripts():
         for ver in RHINO_VERSIONS:
             scripts_folder = os.path.join(home_folder, 'Library', 'Application Support',
                                           'McNeel', 'Rhinoceros', ver, 'scripts')
-            if os.path.isdir(scripts_folder):
-                installed_scripts.append(scripts_folder)
+            if not os.path.isdir(scripts_folder):
+                os.makedirs(scripts_folder)
+            installed_scripts.append(scripts_folder)
     return installed_scripts
 
 
